#include "lmath_fft.h"
#include "lmath_window.h"
#include "lmath_spectrum.h"
#include <stdlib.h>
#include <string.h>


t_lmath_errs lmath_amp_pha_spectrum_peak(const double *src, int size, double dt, int flags,
                                         t_lmath_phase_mode pha_mode,
                                         double *amp, double *pha, double *df) {
    t_lmath_errs err = LMATH_ERR_OK;
    t_lmath_fft_r2c_ctx ctx = lmath_fft_r2c_ctx_create(size, flags);
    if (ctx == NULL) {
        err = LMATH_ERR_MEMORY_ALLOC;
    } else {
        memcpy(lmath_fft_r2c_ctx_in_buf(ctx), src, sizeof(double)*size);
        err = lmath_fft_r2c_calc_amp_pha_spectrum_peak(ctx, pha_mode, amp, pha);
        if (df != NULL)
            *df = lmath_fft_r2c_ctx_df(ctx, dt);
        lmath_fft_r2c_ctx_destroy(ctx);
    }

    return err;
}

t_lmath_errs lmath_amp_pha_spectrum_rms(const double *src, int size, double dt, int flags,
                                        t_lmath_phase_mode pha_mode,
                                        double *amp, double *pha, double *df) {
    t_lmath_errs err = LMATH_ERR_OK;
    t_lmath_fft_r2c_ctx ctx = lmath_fft_r2c_ctx_create(size, flags);
    if (ctx == NULL) {
        err = LMATH_ERR_MEMORY_ALLOC;
    } else {
        memcpy(lmath_fft_r2c_ctx_in_buf(ctx), src, sizeof(double)*size);
        err = lmath_fft_r2c_calc_amp_pha_spectrum_rms(ctx, pha_mode, amp, pha);
        if (df != NULL)
            *df = lmath_fft_r2c_ctx_df(ctx, dt);
        lmath_fft_r2c_ctx_destroy(ctx);
    }

    return err;
}



t_lmath_errs lmath_find_peak_freq(const double *src, int size, double dt, double search_freq,
                                  t_lmath_wintype wintype, double *peak_freq, double *peak_pwr) {
    t_lmath_errs err = LMATH_ERR_OK;
    double *proc_vals = malloc(sizeof(proc_vals[0])*size);
    t_lmath_window_ctx winctx = lmath_window_ctx_create(wintype, 0, size);
    double df = 0.;


    if ((proc_vals == NULL) || (winctx ==NULL)) {
        err = LMATH_ERR_MEMORY_ALLOC;
    } else {
        /* амплитуду берем только на заданной частоте по спектру */
        err = lmath_window_apply_scaled(src, winctx, proc_vals);
    }

    if (!err) {
        err = lmath_amp_pha_spectrum_peak(proc_vals, size, dt, 0, LMATH_PHASE_MODE_COS, proc_vals, 0, &df);
    }

    if (!err) {
        err = lmath_spectrum_find_peak_freq(proc_vals, size/2+1, df, lmath_window_params(winctx),
                                            search_freq, peak_freq, peak_pwr);
    }

    free(proc_vals);
    lmath_window_ctx_destroy(winctx);

    return err;
}
